open Ast

let getOneTerm term =
	match term with
	| Int(x) -> string_of_int(x)
	| String(x) -> x
	| Var(x) -> x
;;

let getLemmaString (lhsPred:Predicate) (tuples:TermList) : string =
	match lhsPred with 
	| Pred(init) ->
	(match tuples with
		| [] -> init
		| _  ->	let stringTuples = List.map getOneTerm tuples in
			let termString = String.concat "," stringTuples in
			(String.concat  ""  (init::"("::termString::[")"])))
;;

let getLiteralString lit =
    match lit with
    | (pred,tuple) -> getLemmaString pred tuple

let getRuleName rule =

    let getRhsLiterals lits =
	String.concat ", " (List.map getLiteralString lits)
    in

    let genConcat sep fn list =
	String.concat "" (List.map (fun x -> String.concat "" (sep::[fn x])) list)
    in

    let getNegLiterals lits =
	genConcat ", ~" getLiteralString lits
    in

    let getFilters filts =
	let getOneFilter filt = match filt with
	| LessThan(t1,t2) -> sprintf "%s&lt;%s" (getOneTerm t1) (getOneTerm t2)
	| GreaterThan(t1,t2) -> sprintf "%s&gt;%s" (getOneTerm t1) (getOneTerm t2)
	| LessThanEqual(t1,t2) -> sprintf "%s&lt;=%s" (getOneTerm t1) (getOneTerm t2)
	| FilterEqual(t1,t2) -> sprintf "%s=%s" (getOneTerm t1) (getOneTerm t2)
	in

	genConcat ", " getOneFilter filts
    in

    let getFunctions fns =
	let rec getExpression e = match e with
	| Plus(e1,e2) -> sprintf "(%s+%s)" (getExpression e1) (getExpression e2)
	| Minus(e1,e2) -> sprintf "(%s-%s)" (getExpression e1) (getExpression e2)
	| Terminal(t) -> sprintf "%s" (getOneTerm t)
	in

	let getOneFunction fn = match fn with
	| FnEqual(x,e) -> sprintf "%s:=%s" (getOneTerm x) (getExpression e)
	in

	genConcat ", " getOneFunction fns
    in

    match rule with
    | Rule(lhsLiteral, rhsPositiveLiterals, rhsNegativeLiterals, filters, functions) ->
	sprintf "%s :- %s%s%s%s"
		(getLiteralString lhsLiteral)
		(getRhsLiterals rhsPositiveLiterals)
		(getFunctions functions)
		(getNegLiterals rhsNegativeLiterals)
		(getFilters filters)
    | _ -> (assert(false)); ""	(* this should never happen *)
;;