open Printf
open System.IO
open Ast
open Transformparseoutput
open Translatetemporaldatalog
open Translaterestrictedtemporaldatalog
open Printutils

(* Reads a file and spits out a [P_Clause]*)
(* Recursion could be infinite, in cases of cyclic use of include <file> directives *)
let rec readFile (filename:string) : (P_Clause List) =
    printf "parsing %s\n" filename;
    try(
		let stream = new StreamReader(filename) in 
	    let myProg = (
			let lexbuf = Lexing.from_text_reader System.Text.Encoding.ASCII stream in 
			try 
				Pars.start Lex.token lexbuf
			with e -> (
			    let pos = lexbuf.EndPos in 
				raise ( new System.Exception("parse error near line " ^ string_of_int(pos.pos_lnum+1) ^ " character " ^ 
				string_of_int(pos.pos_cnum - pos.pos_bol) ^ "; ") )
			)
		)
	    in(
			try
				match myProg with
				| PProg(Filenames(r), b) -> (
					let includeEdbs = List.flatten (List.map readFile r) in
					(List.append includeEdbs b)
				)
			with e -> (
				raise (new System.Exception("Parse error. Check for cyclic includes and normal parse errors",e)  )
			)
		)
    )
    with e -> (
		raise e;
    )
;;

let doSATTranslation(fileName:string) =
(
	try 
    let x = readFile fileName in 
    (
		let validate = validateAll x in 
		let predicateNameToArity = checkPredicateNaming x in
		let filename = "output.txt" in 
		let output = new StreamWriter(filename) in  
		let edbpreds = findandtestEDBPredicates x in 
		let program = transformParseOutToProgram x in 
		//let translatedRules = translateTemporalRulesToPlainRules (program,edbpreds) in
		let translatedRules = translateTempClausesToRules (program, edbpreds) in 
		(
			printf "Validation output: %a\n" output_any ( validate );
			printf "Predicate Naming Map: %a\n" output_any ( predicateNameToArity ) ;
			printf "Unary EDB predicates: %a\n" output_any ( edbpreds ) ;
			prettyPrintProgram ((translatedRules), output);
			output.Close()
		);
		//let t_temp = read_line() in
		//()
    )
    with e -> 
    (
		printf "%s\n" e.Message ; 
		let t_temp = read_line() in
		()
    )

)
let doNegationRestrictedTranslation((fileName:string),(outFileName:string)) =
(
	try 
    let x = readFile fileName in 
    (
		let validate = validateAll x in 
		let predicateNameToArity = checkPredicateNaming x in 
		let output = new StreamWriter(outFileName) in  
		let edbpreds = findandtestEDBPredicates x in 
		let program = transformParseOutToProgram x in
		match (translateNegationRestrictedDatalog(program , edbpreds)) with
		(translatedRules,rFacts,predicateIndexMap) ->
		(
			printf "Validation output: %a\n" output_any ( validate );
			printf "Predicate Naming Map: %a\n" output_any ( predicateNameToArity ) ;
			printf "Unary EDB predicates: %a\n" output_any ( edbpreds ) ;
			Printutils.prettyPrintProgram (translatedRules,output);
			Translaterestrictedtemporaldatalog.prettyPrintInitialFacts(rFacts,output);
			output.Close();
		);
    )
    with e -> 
    (
		printf "%s\n" e.Message ; 
    )
)

let main() = 
    if Array.length Sys.argv <> 3 then begin
	printf "Usage: eon.exe <file> <outputFile>\n";
	exit 1;
    end;
	//doSATTranslation Sys.argv.(1);
	doNegationRestrictedTranslation (Sys.argv.(1) , Sys.argv.(2));
;;

do main()
//let _ =  printf "Press any key to exit..."
//let x = read_line()

(*
    let (flag:string) = Sys.argv.(2) in
    let (fl,rl,ql) = Read.getRules Sys.argv.(1) in 

    Ninterp.saturate fl rl;
    Output.printQueryResults ql flag
	Sat.symbolicExec fl rl
*)
